// main.c
// modified after picocom.c
// to fit the needs for HaD project
// [skaarj] 2016

/* vi: set sw=4 ts=4:
 *
 * picocom.c
 *
 * simple dumb-terminal program. Helps you manually configure and test
 * stuff like modems, devices w. serial ports etc.
 *
 * by Nick Patavalis (npat@efault.net)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <assert.h>
#include <stdarg.h>
#include <signal.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <limits.h>
#include <curses.h>
#include <form.h>



#define _GNU_SOURCE
#include <getopt.h>

#include "term.h"

#include "wits.c"


#define STO STDOUT_FILENO
#define STI STDIN_FILENO

struct {
	char port[128];
	int baud;
	enum flowcntrl_e flow;
	char *flow_str;
	enum parity_e parity;
	char *parity_str;
	int databits;
	int noinit;
	int noreset;
#ifdef UUCP_LOCK_DIR
	int nolock;
#endif
	unsigned char escape;
	char send_cmd[128];
	char receive_cmd[128];
} opts = {
	.port = "",
	.baud = 115200,
	.flow = FC_NONE,
	.flow_str = "none",
	.parity = P_NONE,
	.parity_str = "none",
	.databits = 8,
	.noinit = 0,
	.noreset = 0,
#ifdef UUCP_LOCK_DIR
	.nolock = 0,
#endif
};

int tty_fd;


void cls() {

printf("[H[2J");

}

/**********************************************************************/

#ifdef UUCP_LOCK_DIR

char lockname[_POSIX_PATH_MAX] = "";

int
uucp_lockname(const char *dir, const char *file)
{
	char *p, *cp;
	struct stat sb;

	if ( ! dir || *dir == '\0' || stat(dir, &sb) != 0 )
		return -1;

	/* cut-off initial "/dev/" from file-name */
	p = strchr(file + 1, '/');
	p = p ? p + 1 : (char *)file;
	/* replace '/'s with '_'s in what remains (after making a copy) */
	p = cp = strdup(p);
	do { if ( *p == '/' ) *p = '_'; } while(*p++);
	/* build lockname */
	snprintf(lockname, sizeof(lockname), "%s/LCK..%s", dir, cp);
	/* destroy the copy */
	free(cp);

	return 0;
}

int
uucp_lock(void)
{
	int r, fd, pid;
	char buf[16];
	mode_t m;

	if ( lockname[0] == '\0' ) return 0;

	fd = open(lockname, O_RDONLY);
	if ( fd >= 0 ) {
		r = read(fd, buf, sizeof(buf)); 
		close(fd);
		/* if r == 4, lock file is binary (old-style) */
		pid = (r == 4) ? *(int *)buf : strtol(buf, NULL, 10);
		if ( pid > 0 
			 && kill((pid_t)pid, 0) < 0 
			 && errno == ESRCH ) {
			/* stale lock file */
			printf("Removing stale lock: %s\n", lockname);
			sleep(1);
			unlink(lockname);
		} else {
			lockname[0] = '\0';
			errno = EEXIST;
			return -1;
		}
	}
	/* lock it */
	m = umask(022);
	fd = open(lockname, O_WRONLY|O_CREAT|O_EXCL, 0666);
	if ( fd < 0 ) { lockname[0] = '\0'; return -1; }
	umask(m);
	snprintf(buf, sizeof(buf), "%04d\n", getpid());
	write(fd, buf, strlen(buf));
	close(fd);

	return 0;
}

int
uucp_unlock(void)
{
	if ( lockname[0] ) unlink(lockname);
	return 0;
}

#endif // UUCP_LOCK_DIR 

/**********************************************************************/

ssize_t
writen_ni(int fd, const void *buff, size_t n)
{
	size_t nl; 
	ssize_t nw;
	const char *p;

	p = buff;
	nl = n;
	while (nl > 0) {
		do {
			nw = write(fd, p, nl);
		} while ( nw < 0 && errno == EINTR );
		if ( nw <= 0 ) break;
		nl -= nw;
		p += nw;
	}
	
	return n - nl;
}

int
fd_printf (int fd, const char *format, ...)
{
	char buf[256];
	va_list args;
	int len;
	
	va_start(args, format);
	len = vsnprintf(buf, sizeof(buf), format, args);
	buf[sizeof(buf) - 1] = '\0';
	va_end(args);
	
	return writen_ni(fd, buf, len);
}

void
fatal (const char *format, ...)
{
	char *s, buf[256];
	va_list args;
	int len;

	term_reset(STO);
	term_reset(STI);
	
	va_start(args, format);
	len = vsnprintf(buf, sizeof(buf), format, args);
	buf[sizeof(buf) - 1] = '\0';
	va_end(args);
	
	s = "\r\nFATAL: ";
	writen_ni(STO, s, strlen(s));
	writen_ni(STO, buf, len);
	s = "\r\n";
	writen_ni(STO, s, strlen(s));

	/* wait a bit for output to drain */
	sleep(1);

#ifdef UUCP_LOCK_DIR
	uucp_unlock();
#endif
	
	exit(EXIT_FAILURE);
}

#define cput(fd, c) do { int cl = c; write((fd), &(cl), 1); } while(0)

int
fd_readline (int fdi, int fdo, char *b, int bsz)
{
	int r;
	unsigned char c;
	unsigned char *bp, *bpe;
	
	bp = b;
	bpe = b + bsz - 1;

	while (1) {
		r = read(fdi, &c, 1);
		if ( r <= 0 ) { r--; goto out; }

		switch (c) {
		case '\b':
			if ( bp > (unsigned char *)b ) { 
				bp--;
				cput(fdo, c); cput(fdo, ' '); cput(fdo, c);
			} else {
				cput(fdo, '\x07');
			}
			break;
		case '\r':
			*bp = '\0';
			r = bp - (unsigned char *)b; 
			goto out;
		default:
			if ( bp < bpe ) { *bp++ = c; cput(fdo, c); }
			else { cput(fdo, '\x07'); }
			break;
		}
	}

out:
	return r;
}

#undef cput

/**********************************************************************/


void
child_empty_handler (int signum)
{
}

void
establish_child_signal_handlers (void)
{
	struct sigaction empty_action;
	
	/* Set up the structure to specify the "empty" action. */
    empty_action.sa_handler = child_empty_handler;
	sigemptyset (&empty_action.sa_mask);
	empty_action.sa_flags = 0;
	
	sigaction (SIGINT, &empty_action, NULL);
	sigaction (SIGTERM, &empty_action, NULL);
}

int
run_cmd(int fd, ...)
{
	pid_t pid;
	sigset_t sigm, sigm_old;

	/* block signals, let child establish its own handlers */
	sigemptyset(&sigm);
	sigaddset(&sigm, SIGTERM);
	sigprocmask(SIG_BLOCK, &sigm, &sigm_old);

	pid = fork();
	if ( pid < 0 ) {
		sigprocmask(SIG_SETMASK, &sigm_old, NULL);
		fd_printf(STO, "*** cannot fork: %s\n", strerror(errno));
		return -1;
	} else if ( pid ) {
		/* father: rfid */
		int r;

		/* reset the mask */
		sigprocmask(SIG_SETMASK, &sigm_old, NULL);
		/* wait for child to finish */
		waitpid(pid, &r, 0);
		/* reset terminal (back to raw mode) */
		term_apply(STI);
		/* check and report child return status */
		if ( WIFEXITED(r) ) { 
			fd_printf(STO, "\r\n*** exit status: %d\r\n", 
					  WEXITSTATUS(r));
			return WEXITSTATUS(r);
		} else {
			fd_printf(STO, "\r\n*** abnormal termination: 0x%x\r\n", r);
			return -1;
		}
	} else {
		/* child: external program */
		int r;
		long fl;
		char cmd[512];

		establish_child_signal_handlers();
		sigprocmask(SIG_SETMASK, &sigm_old, NULL);
		/* unmanage terminal, and reset it to canonical mode */
		term_remove(STI);
		/* unmanage serial port fd, without reset */
		term_erase(fd);
		/* set serial port fd to blocking mode */
		fl = fcntl(fd, F_GETFL); 
		fl &= ~O_NONBLOCK;
		fcntl(fd, F_SETFL, fl);
		/* connect stdin and stdout to serial port */
		close(STI);
		close(STO);
		dup2(fd, STI);
		dup2(fd, STO);
		{
			/* build command-line */
			char *c, *ce;
			const char *s;
			int n;
			va_list vls;
			
			c = cmd;
			ce = cmd + sizeof(cmd) - 1;
			va_start(vls, fd);
			while ( (s = va_arg(vls, const char *)) ) {
				n = strlen(s);
				if ( c + n + 1 >= ce ) break;
				memcpy(c, s, n); c += n;
				*c++ = ' ';
			}
			va_end(vls);
			*c = '\0';
		}
		/* run extenral command */
		fd_printf(STDERR_FILENO, "%s\n", cmd);
		r = system(cmd);
		if ( WIFEXITED(r) ) exit(WEXITSTATUS(r));
		else exit(128);
	}
}

/**********************************************************************/

#define TTY_Q_SZ 256

struct tty_q {
	int len;
	unsigned char buff[TTY_Q_SZ];
} tty_q;

/**********************************************************************/




void wits_process(unsigned char buffer[4096], int length) {

	//printf("\nbuf = %s; length=%d\n", buffer, length);  // il bag pe ecran
        int i=0; // cu asta ma deplasez
	int j=0; // frecusbeleala
	int k=0;
	int l=0;

	int modif = 0;
	int cat = 0;
	int index = 0;
	int val = 0;

        char temp[16] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
	char hotdog[5] = {0, 0, 0, 0, 0};

			//printf("%c", buffer[i]);

			// pe pozitia 0 am balarii
		        // poz 1 si 2  &&
		        // poz 3, 4, 5 - WITS
		        // dupa aia incepe scandalu
		        // total recoarde:  (length - 7) / 16

			// se incepe cu o cifra
			// urmeaza ">", 4 cifre, >, 4 cifre, >, 4 cifre .  total 16


	 for (i=7; i<length; i = i + 16) {
			for (j=0; j<16; j++)  temp[j] = buffer[i+j];
			temp[j] = '\0';
			//printf("\ntemp = %s\n", temp);  // am spart carnatu inapoi in categorii wits

			// scot categoria
			modif = temp[0] - 0x30;
			//printf("modif = %d  ", modif);

			k=0;

			for (l=2; l<6; l++) {
						hotdog[k]=temp[l];
						k++;
			}
			hotdog[k]='\0';
			
			sscanf(hotdog, "%X", &cat);
			//printf("cat = %d  ", cat);
			bzero(&hotdog, 5);


			// scot indexu

			k=0;
			for (l=7; l<11; l++) {
						hotdog[k]=temp[l];
						k++;
			}
			hotdog[k]='\0';

			sscanf(hotdog, "%X", &index);
			//printf("index = %d  ", index);
			bzero(&hotdog, 5);


			// scot valoarea
			k=0;
			for (l=12; l<16; l++) {
						hotdog[k] = temp[l];
						k++;
			}
			hotdog[k]='\0';
			sscanf(hotdog, "%X", &val);
			//printf("val = %d \n", val);
			bzero(&hotdog, 5);
			bzero(&temp, 16);

			
// FUCK C COMPILER FOR NOT LETTING ME USE  "SWITCH - CASE" HERE {

					if (cat == sysctl.cat_pwr_ctrl) {
						sysctl.index0[index] = val;
						sysctl.index0_modif[index] = modif;
					}

					if (cat == sysctl.cat_measurements) {
						sysctl.index1[index] = val;
						sysctl.index1_modif[index] = modif;
					}
			
					if (cat == sysctl.cat_manual_input_state) {
						sysctl.index2[index] = val;
						sysctl.index2_modif[index] = modif;
					}
				
					if (cat ==  sysctl.cat_autowarning) {
						sysctl.index3[index] = val;
						sysctl.index3_modif[index] = modif;
					}
		
					if (cat == sysctl.cat_ambient_params) {
						sysctl.index4[index] = val;
						sysctl.index4_modif[index] = modif;
					}

					if (cat == sysctl.cat_TBD) {
					}				


//	} // END FUCK

/*			if (cat == 0x00)  {   // cat 0 -> power controls
			//	for (j = 2; j<7; j++);    // scot indexu
						
					
			}

			if (cat == 0x01) {   // cat 1 -> measurements

			}

			if (cat = 0x02) {  // cat 2 -> manual input states
			}

			if (cat = 0x03) {  // cat 3 -> autimatic warnings
			}
			
			if (cat = 0x04) {  // cat 4 -> ambient params


			}

			if (cat == 0x05) {  // cat 5 -> to be determined

			}
					
*/			
			


          } // end for

		print_wits(buffer, length);



}



/**********************************************************************/

void loop(void)
{
	int dtr_up;
	fd_set rdset, wrset;
	int i,n;
	unsigned char buf[4096];
	unsigned char c='\0';
	int buflen = 512;
	int sync = 0;
	int end_rec_warning = 0;

	bzero(&buf, buflen);
	tty_q.len = 0;
	dtr_up = 0;

	//printf("\nSyncing...\n");
	bzero(&buf, buflen);

	while (1) {  
	 		        FD_ZERO(&rdset);
                                FD_ZERO(&wrset);                                
                                FD_SET(STI, &rdset);   
                                FD_SET(tty_fd, &rdset);
				if (select(FD_SETSIZE, &rdset, &wrset, NULL, NULL) < 0)
                                                                fatal("select failed: %d : %s", errno, strerror(errno));

                                if ( FD_ISSET(tty_fd, &rdset) ) {

							       do { 
								    n = read(tty_fd, &c, 1);
   								    if (sync == 0) {
										     // citesc pana dau de "S" de la WITS
										    if (c == 'S') { 
												sync = 1;  // daca l-am gasit, gata sincronizarea
												//printf("\nS Y N C E D:  buf = %c.\n", c);
												i=0;
											        bzero(&buf, 4096);
										    }
								    }
								
								   if (sync == 1) {  // suntem sincronizati, procesam
  
 								    	       // daca caracteru nu e linie noua, il bag la buf
									
                                                                    		if (c == 0x0d) c = '#';
										if (c == 0x0a) c = '#'; 

										if ((c != '!') && (c != '#'))  {   // daca nu e end of record nici linie noua...
	                                                                        	buf[i] = c;
									        	i++;
										}


										if (c== '!') end_rec_warning++;

										if (end_rec_warning == 2) {  // daca primesc <end of record>
												 
												//buf[i] = '\0';  // completez cu end of string
												if (buf[1] == '&') wits_process(buf, i);
												//if (buf[1] == '&') printf("buf = %s.\n", buf);
												bzero(&buf, 4096); //fac zero pe tot carnatu
												i=0; // merg inapoi pe prima pozitie
												end_rec_warning = 0;
										}

								   }
							
								} while ((n < 0) && (errno == EINTR));
							

								if (n == 0)
									fatal("term closed");
								else if ( n < 0 )
									fatal("read from term failed: %s", strerror(errno));
			
								if ( n <= 0 )
									fatal("write to stdout failed: %s", strerror(errno));
				}		

	}

}

/**********************************************************************/


void
deadly_handler(int signum)
{
	kill(0, SIGTERM);
	sleep(1);
#ifdef UUCP_LOCK_DIR
	uucp_unlock();
#endif
	exit(EXIT_FAILURE);
}


void
establish_signal_handlers (void)
{
        struct sigaction exit_action, ign_action;

        /* Set up the structure to specify the exit action. */
        exit_action.sa_handler = deadly_handler;
        sigemptyset (&exit_action.sa_mask);
        exit_action.sa_flags = 0;

        /* Set up the structure to specify the ignore action. */
        ign_action.sa_handler = SIG_IGN;
        sigemptyset (&ign_action.sa_mask);
        ign_action.sa_flags = 0;

        sigaction (SIGTERM, &exit_action, NULL);

        sigaction (SIGINT, &ign_action, NULL); 
        sigaction (SIGHUP, &ign_action, NULL);
        sigaction (SIGALRM, &ign_action, NULL);
        sigaction (SIGUSR1, &ign_action, NULL);
        sigaction (SIGUSR2, &ign_action, NULL);
        sigaction (SIGPIPE, &ign_action, NULL);
}

/**********************************************************************/

/**********************************************************************/

void
parse_args()	{

		        opts.noinit=0;
		        opts.noreset=0;
		        opts.nolock=0;
		        opts.baud=115200;
		        opts.parity_str = "none";
		        opts.parity = P_NONE;
		        opts.databits=8;

		        opts.flow_str="none";
		        opts.flow=FC_NONE;
        
		        strncpy(opts.port, "/dev/cuaU0\0",sizeof(opts.port) -1);
			opts.port[sizeof(opts.port) - 1] = '\0';

			printf("vasea's tool for reading fucking WITS data.\n");
			printf("\n");
			printf("port is        : %s\n", opts.port);
			printf("flowcontrol    : %s\n", opts.flow_str);
			printf("baudrate is    : %d\n", opts.baud);
			printf("parity is      : %s\n", opts.parity_str);
			printf("databits are   : %d\n", opts.databits);
			printf("escape is      : C-%c\n", 'a' + opts.escape - 1);
			printf("noinit is      : %s\n", opts.noinit ? "yes" : "no");
			printf("noreset is     : %s\n", opts.noreset ? "yes" : "no");
			printf("nolock is      : %s\n", opts.nolock ? "yes" : "no");
			printf("send_cmd is    : %s\n", opts.send_cmd);
			printf("receive_cmd is : %s\n", opts.receive_cmd);
			printf("\n");

}

/**********************************************************************/


WINDOW *mainwin = NULL;
WINDOW *win_cat0 = NULL;
WINDOW *win_cat1 = NULL;
WINDOW *win_cat2 = NULL;
WINDOW *win_cat3 = NULL;
WINDOW *win_cat4 = NULL;

int row=0, col=0;
int mincol = 0;
int total_strlen = 0;
int ch = 0;
int attr = A_BOLD;
char msg_copyright1[] = "DACIA 1310 WITS DIAGNOSIS\0";
char msg_copyright2[] = "(C) 2016 [skaarj]\0";
char msg_win0[] = "PWR CTRL\0";
char msg_win1[] = "MEASURES\0";
char msg_win2[] = "USR INPUTS\0";
char msg_win3[] = "AUTO INPUTS\0";
char msg_win4[] = "AMBIENT\0";

int clear_win() {   // inchidere mod curses pentru iesire din program, restaurare mod text initial
                refresh();
                refresh();
                endwin();
                return 0;
}


int testmenu(void) {  // verificare rezolutie terminal - incape interfata sau trebuie marit ecranul?
             
                char err_resize_screen[] = "Resize screen";
                getmaxyx(stdscr, row, col);  // total linii, total coloane
             
                if (has_colors() == false) {
                                                endwin();
                                                printf("Warning, no color!\n");
                                                exit(EXIT_FAILURE);
                }

                
                mincol = 100;
                if (mincol > col) {
                                        mvprintw(row/2, (col-strlen(err_resize_screen)/2), "%s", err_resize_screen);
                                        clear_win();
                                        return -1;
                                        exit(EXIT_FAILURE);
                }
return 0;
}


int initcurses() {

                // initializare ecran pentru modul curses
                // trecere ecran in mod coordonate
                // schimbare atribute: text alb, fundal negru
        
                 if ( initscr() == NULL ) {
                                                clear_win();
                                                fprintf(stderr, " Error - ncurses_init() failed\n");
                                                exit(EXIT_FAILURE);
                }
        
             testmenu();
             //wtimeout(mainwin,0);
             raw();
             clear();
             noecho();
             curs_set(0);
             cbreak();
             start_color();
             // use_default_colors();   culorile puse la dispozitie de terminal -> atentie nu se vede pe fundal alb!!
             refresh();
        
             getmaxyx(stdscr, row, col);  // total linii, total coloane
        
             mainwin = newwin(4, col, 0, 0);
	     win_cat0 = newwin((row-row/8), col/5, 4, 0);
	     win_cat1 = newwin(row-row/8, col/5, 4, col/5);
	     win_cat2 = newwin(row-row/8, col/5, 4, 2*col/5);
	     win_cat3 = newwin(row-row/8, col/5, 4, 3*col/5);
	     win_cat4 = newwin(row-row/8, col/5, 4, 4*col/5);
	    

             refresh();
             box(mainwin, 0, 0);      // chenare in fiecare din ferestrele definite, incepand din coltul stanga-sus al fiecarei ferestre
	     box(win_cat0, 0, 0);
	     box(win_cat1, 0, 0);
	     box(win_cat2, 0, 0);
	     box(win_cat3, 0, 0);
	     box(win_cat4, 0, 0);
            
                // definire perechi de culori:   nr. pereche, culoare text, culoare fundal
                init_pair(1, COLOR_WHITE, COLOR_BLACK);     // normal: alb pe negru
                init_pair(2, COLOR_GREEN, COLOR_GREEN);     // status OK:  verde pe verde
                init_pair(3, COLOR_RED, COLOR_RED);         // status warning:  rosu pe rosu
                init_pair(4, COLOR_GREEN, COLOR_BLACK);     // axe de coordonate, parametri in limite normale:  verde pe negru
                init_pair(5, COLOR_RED, COLOR_BLACK);       // parametri in cota de poluare:  rosu pe negru
                init_pair(6, COLOR_YELLOW, COLOR_BLACK);    // parametri in cota de atentie: galben pe negru
                init_pair(7, COLOR_YELLOW, COLOR_YELLOW);   // status atentie:  galben pe galben
                
                /* mainwin -> text titlu program
                
                mvwprintw   (descriere:  man mvwprintw)
                sintaxa:   int mvwprintw ( WINDOW *win, int coordonata_y,  int coordonata_x,   const char *format, variabile.......)
                
                */
                mvwprintw(mainwin, 1, (col-strlen((char *)msg_copyright1))/2, "%s", (char *)msg_copyright1);
                mvwprintw(mainwin, 2, (col-strlen((char *)msg_copyright2))/2, "%s", (char *)msg_copyright2);

               // dupa ce am schimbat in fereastra, se face update la fereastra cu wrefresh()
                wrefresh(mainwin);
		wrefresh(win_cat0);
		wrefresh(win_cat1);
		wrefresh(win_cat2);
		wrefresh(win_cat3);
		wrefresh(win_cat4);
		return 0;
}


int attrib_on (WINDOW *win, unsigned int status) {
                                                        if (status == 0) wattron(win, COLOR_PAIR(4));
                                                        if (status == -1) wattron(win, COLOR_PAIR(6));
                                                        if (status == 1) wattron(win, COLOR_PAIR(5));

return 0;
}


int attrib_off (WINDOW *win, unsigned int status) {
                                                        if (status == 0) wattroff(win, COLOR_PAIR(4));
                                                        if (status == -1) wattroff(win, COLOR_PAIR(6));
                                                        if (status == 1) wattroff(win, COLOR_PAIR(5));
return 0;
}


int print_wits(unsigned char buf[4096], int len) {
		int i=0;

		//mvwprintw(mainwin, 0, 0, "%s", buf);
		mvwprintw(win_cat0, 2, (col/5 - strlen ((char *)msg_win0))/4, "%s", msg_win0);
		mvwprintw(win_cat1, 2, (col/5 - strlen ((char *)msg_win1))/4, "%s", msg_win1);
		mvwprintw(win_cat2, 2, (col/5 - strlen ((char *)msg_win2))/4, "%s", msg_win2);
		mvwprintw(win_cat3, 2, (col/5 - strlen ((char *)msg_win3))/4, "%s", msg_win3);
		mvwprintw(win_cat4, 2, (col/5 - strlen ((char *)msg_win4))/4, "%s", msg_win4);
		

		for (i=0; i<sysctl.num_idx0; i++) {
			attrib_on(win_cat0, sysctl.index0_modif[i]);
			mvwprintw(win_cat0, i+4, 2, "%2d %s: %2d", i, sysctl.m0[i], sysctl.index0[i]);
			attrib_off(win_cat0, sysctl.index0_modif[i]);
		}


		for (i=0; i<sysctl.num_idx1; i++) {
			attrib_on(win_cat1, sysctl.index1_modif[i]);
			mvwprintw(win_cat1, i+4, 2, "%2d %s: %3d", i, sysctl.m1[i], sysctl.index1[i]);
			attrib_off(win_cat1, sysctl.index1_modif[i]);
			attrib_off(win_cat1, sysctl.index1_modif[i]);			
		}

		for (i=0; i<sysctl.num_idx2; i++) {
			attrib_on(win_cat2, sysctl.index2_modif[i]);
			mvwprintw(win_cat2, i+4, 2, "%2d %s: %3d", i, sysctl.m2[i], sysctl.index2[i]);
			attrib_off(win_cat2, sysctl.index2_modif[i]);
		}

		for (i=0; i<sysctl.num_idx3; i++) {
			attrib_on(win_cat3, sysctl.index3_modif[i]);
			mvwprintw(win_cat3, i+4, 2, "%2X %s: %3X", i, sysctl.m3[i], sysctl.index3[i]);
			attrib_off(win_cat3, sysctl.index3_modif[i]);
		}

		for (i=0; i<sysctl.num_idx4; i++) {
			attrib_on(win_cat4, sysctl.index4_modif[i]);
			mvwprintw(win_cat4, i+4, 2, "%2d %s: %3d", i, sysctl.m4[i], sysctl.index4[i]);
			attrib_off(win_cat4, sysctl.index4_modif[i]);
		}

		//wrefresh(mainwin);
		wrefresh(win_cat0);
		wrefresh(win_cat1);
		wrefresh(win_cat2);
		wrefresh(win_cat3);
		wrefresh(win_cat4);

return 0;
}


int main() {
		int r;
		parse_args();
		initcurses();
		witsinit();

		establish_signal_handlers();

		r = term_lib_init();
		if ( r < 0 )
			fatal("term_init failed: %s", term_strerror(term_errno, errno));

		#ifdef UUCP_LOCK_DIR
				if ( ! opts.nolock ) uucp_lockname(UUCP_LOCK_DIR, opts.port);
				if ( uucp_lock() < 0 )
				fatal("cannot lock %s: %s", opts.port, strerror(errno));
		#endif


		tty_fd = open(opts.port, O_RDWR | O_NONBLOCK);
		if (tty_fd < 0)
				fatal("cannot open %s: %s", opts.port, strerror(errno));

		if ( opts.noinit ) {
					r = term_add(tty_fd);
		} else {
			r = term_set(tty_fd,
					 1,              /* raw mode. */
					 opts.baud,      /* baud rate. */
					 opts.parity,    /* parity. */
					 opts.databits,  /* data bits. */
					 opts.flow,      /* flow control. */
					 1,              /* local or modem */
					 !opts.noreset); /* hup-on-close. */
		}
		if ( r < 0 )
			fatal("failed to add device %s: %s", 
				  opts.port, term_strerror(term_errno, errno));
		r = term_apply(tty_fd);
		if ( r < 0 )
			fatal("failed to config device %s: %s", 
				  opts.port, term_strerror(term_errno, errno));
		r = term_add(STI);
		if ( r < 0 )
			fatal("failed to add I/O device: %s", 
				  term_strerror(term_errno, errno));
		term_set_raw(STI);
		r = term_apply(STI);
		if ( r < 0 )
			fatal("failed to set I/O device to raw mode: %s",
					  term_strerror(term_errno, errno));

	        //fd_printf(STO, "# ");

	        loop();


		//fd_printf(STO,"  ");
		//	fd_printf(STO, "  \r");
		if ( opts.noreset ) {
			fd_printf(STO, "Skipping tty reset...\r\n");
			term_erase(tty_fd);
		}

		//	fd_printf(STO, "exit().\r\n");
		/* wait a bit for output to drain */
			sleep(1);


	     	sleep(5);
	      	clear_win();
		delwin(mainwin);
		endwin();

			#ifdef UUCP_LOCK_DIR
				uucp_unlock();
			#endif
			return EXIT_SUCCESS;
		
}


